import asyncio
import random

import flet as ft

import flet_easy as fs

app = fs.FletEasy()


# remove animation on route change
@app.config
def config(page: ft.Page):
    page.theme = ft.Theme(
        page_transitions=ft.PageTransitionsTheme(
            windows=ft.PageTransitionTheme.NONE,
            android=ft.PageTransitionTheme.NONE,
            ios=ft.PageTransitionTheme.NONE,
            macos=ft.PageTransitionTheme.NONE,
            linux=ft.PageTransitionTheme.NONE,
        ),
    )


# use in 'data.view'
@app.view
def config_view(data: fs.Datasy):
    def change_color(e: ft.ControlEvent):
        colors = [ft.Colors.RED, ft.Colors.GREEN, ft.Colors.BLUE, ft.Colors.YELLOW]
        appbar.bgcolor = random.choice(colors)
        data.page.update()

    appbar = ft.AppBar(
        bgcolor=ft.Colors.BLACK45,
        actions=[
            ft.IconButton(
                icon=ft.Icons.RESTART_ALT_ROUNDED,
                icon_color=ft.Colors.WHITE,
                on_click=change_color,
            )
        ],
    )

    def index_bar_route(e: ft.ControlEvent):
        route = e.control.selected_index

        if route == 0:
            data.go("/")()
        elif route == 1:
            data.go("/test2")()
        elif route == 2:
            data.go("/test3")()

    return ft.View(
        appbar=appbar,
        navigation_bar=ft.NavigationBar(
            destinations=[
                ft.NavigationBarDestination(icon=ft.Icons.CODE, label="counter 1"),
                ft.NavigationBarDestination(icon=ft.Icons.SYNC_DISABLED, label="counter 2"),
                ft.NavigationBarDestination(
                    icon=ft.Icons.CODE,
                    label="counter 3",
                ),
            ],
            on_change=index_bar_route,
        ),
    )


# control custom
class Counter(ft.Container):
    def __init__(self, update, color: str):
        super().__init__()
        self.update = update

        self.number = ft.TextField(value="0", text_size=50, text_align="center")
        self.content = ft.Column(
            controls=[
                self.number,
                ft.FilledButton("start", on_click=self.start, height=50),
            ],
            horizontal_alignment="center",
        )
        self.width = 400
        self.bgcolor = color
        self.border_radius = 10
        self.padding = 20

    async def start(self, e):
        while True:
            self.number.value = str(int(self.number.value) + 1)
            self.update()
            await asyncio.sleep(1)


# add cache to the page
@app.page("/", title="Test 1", cache=True)
def index_page(data: fs.Datasy):
    page = data.page
    new_appbar = fs.cache_control(data.view.appbar)
    new_appbar.title = ft.Text("Test 1")

    return ft.View(
        controls=[
            ft.Text("Counter 1", size=50),
            Counter(page.update, ft.Colors.RED),
        ],
        appbar=new_appbar,
        navigation_bar=data.view.navigation_bar,
        horizontal_alignment="center",
        vertical_alignment="center",
    )


def m(data: fs.Datasy):
    page = data.page
    page.run_task(x)


@app.page("/test2", title="Test 2", middleware=[m])
def test_page(data: fs.Datasy):
    page = data.page
    data.view.appbar.title = ft.Text("Test 2")

    return ft.View(
        controls=[
            ft.Text("Counter 2 - (cache disabled)", size=50),
            Counter(page.update, ft.Colors.BLUE),
        ],
        appbar=data.view.appbar,
        navigation_bar=data.view.navigation_bar,
        vertical_alignment="center",
        horizontal_alignment="center",
    )


async def x():
    for i in range(10):
        print(i)
        await asyncio.sleep(1)
    return "Done"


# add cache to the page
@app.page("/test3", title="Test 3", cache=True)
async def test2_page(data: fs.Datasy):
    page = data.page
    new_appbar = fs.cache_control(data.view.appbar)
    new_appbar.title = ft.Text("Test 3")

    page.run_task(x)

    return ft.View(
        controls=[
            ft.Text("Counter 3", size=50),
            Counter(page.update, ft.Colors.GREEN),
        ],
        appbar=new_appbar,
        navigation_bar=data.view.navigation_bar,
        horizontal_alignment="center",
        vertical_alignment="center",
    )


async def main(page: ft.Page):
    app.start(page)


# app.run()
if __name__ == "__main__":
    #ft.app(target=main)
    app.run()
